package org.globus.transfer.reliable.service.test.client;

import junit.framework.TestCase;
import javax.xml.namespace.QName;
import org.globus.rft.generated.TransferType;
import org.globus.wsrf.ResourcePropertySet;
import org.globus.wsrf.impl.SimpleResourcePropertySet;
import org.globus.wsrf.impl.timer.TimerManagerImpl;
import org.globus.transfer.reliable.service.exception.RftException;
import org.globus.transfer.reliable.service.RFTResourceManager;
import org.globus.transfer.reliable.service.cache.ConnectionCacheConfiguration;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.client.ClientFactory;

import commonj.timers.TimerManager;

public class ClientFactoryTest extends TestCase
{
    private TimerManager timerManager;
    private ConnectionManager connManager;
    private RFTResourceManager resourceManager;
    private ResourcePropertySet rpSet;
    private TransferType transferJob;
    private int transferId;
    
	protected void setUp() throws Exception
	{
		super.setUp();
		String namespaceURI = "http://www.globus.org/test";
		String localName = "ClientFactoryTest";
		QName myQName= new QName(namespaceURI,localName);
		this.rpSet = new SimpleResourcePropertySet(myQName);
		
		this.resourceManager
		    = new RFTResourceManager(
		        42, 1, new SimpleResourcePropertySet(myQName));
		
        this.timerManager = new TimerManagerImpl();
        ConnectionCacheConfiguration cacheConfig = new ConnectionCacheConfiguration();
        cacheConfig.setConnectionTimeout(5000);
        cacheConfig.setMaxNumberCachedFDs(10);
        cacheConfig.setSweeperInterval(1000);

        this.connManager =  ConnectionManager.createNewInstance(
            cacheConfig, timerManager);

        this.transferJob = new TransferType();
        this.transferId = 42;
	}
	
	protected void tearDown() throws Exception {
	    this.connManager.closeAllConnections();
	}

	/**
	 * Test for the helper function getGlobusURL(String)
	 */ 
	public void testGetGlobusUrl()
	{
		String[] failureStrings = new String[5];
		String[] okStrings = new String[1];
		failureStrings[0] = null;
		failureStrings[1] = "";
		failureStrings[2] = "globus";
		failureStrings[3] = "http://";
	    failureStrings[4] = "http://www.globus.org/";
	    okStrings[0] = "gsiftp://www.globus.org:2811/myFile";
	    
	    // check strings that should cause failures
	    for(int i=0; i<failureStrings.length; i++) {
	        try {
	            ClientFactory.getGlobusURL(failureStrings[i]);
                fail("An exception should be thrown for url string " +
                     failureStrings[i]);
	        } catch (RftException e1) {
	    	    // this is what we expect
	        } catch (Exception e2) {
	    	    fail("An RftException should be thrown for url string " +
	    	         failureStrings[i]);
	        }
	    }
	    
        // check strings that should work
	    for(int i=0; i<okStrings.length; i++) {
	        try {
	            ClientFactory.getGlobusURL(okStrings[i]);
	        } catch (Exception e) {
		    	fail(okStrings[i] + " should have been ok for " +
		             "creating a GlobusURL");
	        }
	    }
	}
	
    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateDeleteClient()
	{
	    // test null parameters
	    
	    // transferJob parameter is null
	    try {
	    	ClientFactory.createDeleteClient(null,
	    	    this.resourceManager, this.connManager);
	    	fail("an RftException should have been thrown here");
	    } catch (RftException e) {
	    } catch (Exception e) {
	        fail("should be an RftException here");
	    }
	    
	    // resourceManager parameter is null
        try {
            ClientFactory.createDeleteClient(this.transferJob, null,
                this.connManager);
            fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here");
        }
        
        // connectionManager parameter is null
        try {
            ClientFactory.createDeleteClient(this.transferJob,
                this.resourceManager, null);
            fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here");
        }	    
	}

    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateGetPutTransferClient()
	{	
		// TransferType parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            null, this.resourceManager, this.connManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
        // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            transferJob, null, this.connManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
        // connectionManager parameter is null
        try {
            ClientFactory.createGetPutTransferClient(
                transferJob, this.resourceManager, null, rpSet, transferId);
            fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
	    // ResourcePropertySet parameter is null
	    try {
	    	ClientFactory.createGetPutTransferClient(
	            transferJob, resourceManager, this.connManager, null, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	}

    /**
     * Tested is only whether the right exceptions are thrown
     * for invalid input
     */
	public void testCreateThirdPartyTransferClient()
	{
		// TransferType parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            null, resourceManager, connManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            transferJob, null, connManager, rpSet, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
        // connectionManager parameter is null
        try {
            ClientFactory.createThirdPartyTransferClient(
                transferJob, resourceManager, null, rpSet, transferId);
            fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
	    // ResourcePropertySet parameter is null
	    try {
	    	ClientFactory.createThirdPartyTransferClient(
	            transferJob, resourceManager, connManager, null, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	}

	public void testCreateURLExpanderClient()
	{
		// TransferType parameter is null
	    try {
	    	ClientFactory.createURLExpanderClient(
	            null, resourceManager, connManager, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
	    // RFTResourceManager parameter is null
	    try {
	    	ClientFactory.createURLExpanderClient(
	            transferJob, null, connManager, transferId);
	    	fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	    
        // connectionManager parameter is null
        try {
            ClientFactory.createURLExpanderClient(
                transferJob, resourceManager, null, transferId);
            fail("an RftException should have been thrown here");
        } catch (RftException e) {
        } catch (Exception e) {
            fail("should be an RftException here and not a " +
                e.getClass().getName());
        }
	}
}
