package org.globus.transfer.reliable.service.test.connection;

import junit.framework.TestCase;
import org.globus.rft.generated.RFTOptionsType;
import org.ietf.jgss.GSSCredential;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.Connection;
import org.globus.transfer.reliable.service.cache.SingleConnectionImpl;
import org.globus.transfer.reliable.service.cache.ConnectionCounter;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.ConnectionCacheImpl;

/**
 * The cache is tested with a connection of type SingleConnectionImpl
 */
public class ConnectionCacheImplTest extends TestCase {
    
    private SingleConnectionImpl connection;
    private ConnectionParameter connectionParameter1;
    private ConnectionParameter connectionParameter2;    
    private ConnectionCacheImpl connectionCache;
    private GSSCredential credential = null;
    private ConnectionCounter connectionCounter;
    private int connectionLifetime;
    
    public ConnectionCacheImplTest(String name) {
        
        super(name);
    }

    protected void setUp() throws Exception {

        super.setUp();

        try {
            RFTOptionsType rftOptions1 = new RFTOptionsType();
            rftOptions1.setDcau(new Boolean(false));
            RFTOptionsType rftOptions2 = new RFTOptionsType();
            rftOptions2.setDcau(new Boolean(true));
            
            // lifetime of a connection 5s
            this.connectionLifetime = 5000;
            
            this.connectionParameter1 = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                TestConfig.sourcePath, TestConfig.subject, rftOptions1);
            this.connectionParameter2 = new ConnectionParameter(
                TestConfig.sourceHost, TestConfig.sourcePort,
                TestConfig.sourcePath, TestConfig.subject, rftOptions2);

            TestConfig.printConfig();
            
            this.credential = new GlobusGSSCredentialImpl(
                GlobusCredential.getDefaultCredential(),
                GSSCredential.INITIATE_AND_ACCEPT);

            this.connection = new SingleConnectionImpl(
                connectionParameter1,credential,null);

            this.connectionCounter = new ConnectionCounter();

            this.connectionCache = new ConnectionCacheImpl(
                ConnectionManager.SINGLE_CONNECTION, this.connectionLifetime,
                this.connectionCounter);

        } catch(Exception e) {
            fail("Failed to create ConnectionCacheImpl: " + e);
            e.printStackTrace();
        }
    }
    
    protected void tearDown() {
        
        this.connectionCache.closeAllConnections();
    }

    public void testGetType() {
        
        assertTrue(this.connectionCache.getType() ==
             this.connection.getType());
    }

    public void testAddConnection() {
        
        assertTrue(this.connectionCounter.getValue() == 0);
        try {
            this.connectionCache.addConnection(this.connection);
        } catch(Exception e) {
            fail("Failed to addConnection: " + e);
        }
        assertTrue(this.connectionCounter.getValue() ==
            this.connection.getNumberFDs());
    }

    public void testGetConnection() {
        
        assertTrue(this.connectionCounter.getValue() == 0);

        // try to get a connection from an empty cache
        try {
            Connection conn = this.connectionCache.getConnection(
                this.connectionParameter1);
            assertTrue(conn == null);
            assertTrue(this.connectionCounter.getValue() == 0);
        } catch(Exception e) {
            fail("Failed to get connection: " + e);
        }
        
        // add a connection and get it from the cache
        try {
            this.connectionCache.addConnection(this.connection);
            assertTrue(this.connectionCounter.getValue() ==
                this.connection.getNumberFDs());
            Connection conn = this.connectionCache.getConnection(
                this.connectionParameter1);
            assertTrue(conn.equals(this.connection));
            assertTrue(this.connectionCounter.getValue() == 0);
        } catch(Exception e) {
            fail("Failed to add/get connection: " + e);
        }
        
        // add a connection and try to get a different one
        try {
            this.connectionCache.addConnection(this.connection);
            assertTrue(this.connectionCounter.getValue() ==
                this.connection.getNumberFDs());
            Connection conn = this.connectionCache.getConnection(
                this.connectionParameter2);
            assertTrue(conn == null);
            assertTrue(this.connectionCounter.getValue() ==
                this.connection.getNumberFDs());
        } catch(Exception e) {
            fail("Failed to add/get connection: " + e);
        }
    }

    public void testShrink() {
        
        assertTrue(this.connectionCounter.getValue() == 0);
        try
        {
            for (int i=1; i<5; i++) {
                this.connectionCache.addConnection(this.connection);
                assertTrue(this.connectionCounter.getValue() == 
                    (i * this.connection.getNumberFDs()));                
            }
            
            for (int i=4; i>0; i--) {
                assertTrue(this.connectionCounter.getValue() == 
                    (i * this.connection.getNumberFDs()));
                this.connectionCache.shrink();
            }
            
            assertTrue(this.connectionCounter.getValue() == 0);
            this.connectionCache.shrink();
            assertTrue(this.connectionCounter.getValue() == 0);
        } catch(Exception e) {
            fail("testShrink() failed: " + e);
        }
    }

    public void testGetOldestTimestamp() {

        assertTrue(this.connectionCache.getOldestTimestamp() == -1);
        try {
            this.connectionCache.addConnection(this.connection);
        } catch(Exception e) {
            fail("Failed to add connection: " + e);
        }
        assertTrue(this.connectionCache.getOldestTimestamp() > -1);
    }

    public void testCloseAllConnections() {

        assertTrue(this.connectionCounter.getValue() == 0);
        try {
            for (int i=1; i<5; i++) {
                this.connectionCache.addConnection(this.connection);
                assertTrue(this.connectionCounter.getValue() == 
                    (i * this.connection.getNumberFDs()));                
            }
            this.connectionCache.closeAllConnections();
            assertTrue(this.connectionCache.getConnection(
                this.connectionParameter1) == null);
            assertTrue(this.connectionCounter.getValue() == 0);
        } catch(Exception e) {
            fail("testCloseAllConnections() failed: " + e);
        }
    }

    public void testRemoveExpiredConnections() {
        
        assertTrue(this.connectionCounter.getValue() == 0);
        try {
            this.connectionCache.addConnection(this.connection);
        } catch(Exception e) {
            fail("Failed to addConnection: " + e);
        }
        assertTrue(this.connectionCounter.getValue() == 
            this.connection.getNumberFDs());
        
        // make sure the connection is expired
        try {
            Thread.sleep((this.connectionLifetime+1000));
        } catch (InterruptedException e) {
            fail("sleep got interrupted");
        }

        this.connectionCache.removeExpiredConnections();

        // the connection should not be found anymore
        Connection conn = this.connectionCache.getConnection(
            this.connectionParameter1);
        assertTrue(conn == null);
        assertTrue(this.connectionCounter.getValue() == 0);        
    }
}
