package org.globus.transfer.reliable.service.test.connection;

import java.io.File;
import java.util.Date;
import junit.framework.TestCase;
import org.globus.ftp.MlsxEntry;
import org.globus.rft.generated.RFTOptionsType;
import org.ietf.jgss.GSSCredential;
import org.globus.gsi.GlobusCredential;
import org.globus.gsi.gssapi.GlobusGSSCredentialImpl;
import org.globus.transfer.reliable.service.test.TestConfig;
import org.globus.transfer.reliable.service.cache.ConnectionManager;
import org.globus.transfer.reliable.service.cache.ConnectionParameter;
import org.globus.transfer.reliable.service.cache.SingleConnectionImpl;
import org.globus.transfer.reliable.service.exception.RftException;
import org.globus.transfer.reliable.service.exception.RftTransientException;

public class SingleConnectionImplTest extends TestCase {
    
    private SingleConnectionImpl conn;
    private RFTOptionsType rftOptions1;
    private RFTOptionsType rftOptions2;
    private ConnectionParameter params1;
    private ConnectionParameter params2;
    private GSSCredential credential;
    private String sourceSubject;
    
    public SingleConnectionImplTest(String name) {
        super(name);
    }

    protected void setUp() throws Exception {
        
        super.setUp();
        this.credential = new GlobusGSSCredentialImpl(
            GlobusCredential.getDefaultCredential(),
            GSSCredential.INITIATE_AND_ACCEPT);

        // create different RFTOptionsType objects
        rftOptions1 = new RFTOptionsType();
        rftOptions1.setDcau(new Boolean(true));
        rftOptions2 = new RFTOptionsType();
        rftOptions2.setDcau(new Boolean(false));

        this.sourceSubject = TestConfig.sourceSubject;
        if (this.sourceSubject == null || this.sourceSubject.trim().equals("")) {
            this.sourceSubject = TestConfig.subject;
        }

        // create different connection parameter objects
        params1 = new ConnectionParameter(
            TestConfig.sourceHost, TestConfig.sourcePort,
            TestConfig.sourcePath, this.sourceSubject,
            rftOptions1);
        params2 = new ConnectionParameter(
            TestConfig.destinationHost, TestConfig.destPort,
            TestConfig.destPath, this.sourceSubject,
            rftOptions2);

        this.conn = new SingleConnectionImpl(params1,credential,
            TestConfig.sourceSubject);
    }
    
    protected void tearDown() {
        
        this.conn.closeConnection();
    }

    public void testCheckConnectionIntegrity() {
       
        try {
            assertTrue(this.conn.checkConnectionIntegrity());
        } catch(Exception e) {
            fail("testCheckConnectionIntegrity(): " + e);
        }
    }

    public void testCheckForReuse() {
        
        try {
            assertTrue(this.conn.checkForReuse(this.params1));
            assertFalse(this.conn.checkForReuse(this.params2));
        } catch(Exception e) {
            fail("testCheckForReuse() failed: " + e);
        }
    }

    public void testGetType() {
        
        try {
            assertTrue(
                this.conn.getType() == ConnectionManager.SINGLE_CONNECTION);
        } catch(Exception e) {
            fail("testGetType() failed: " + e);
        }
    }

    public void testGetNumberFDs() {
        
        try {
            assertTrue(this.conn.getNumberFDs() == 1);
        } catch(Exception e) {
            fail("testGetNumberFDs() failed: " + e);
        }
    }
    
    public void testPermissions() {
        
        try {
            TestUtil.createTestDir(params1.getSourceHost(),
                params1.getSourcePort(), this.sourceSubject, credential);
            File dir = new File(TestUtil.REMOTE_TEST_DIR);
            
            try {
                this.conn.setPath(dir.getAbsolutePath());
                this.conn.setPermissions(null);
                fail("setting null permissions should cause an RftException");
            } catch(RftException e) {
            } catch(Exception e) {
                fail("setting null permissions should cause an RftException "
                    + "and not a " + e.getClass().getName());
            }

            MlsxEntry mEntry = this.conn.doMlst();
            System.out.println("old MlsxEntry: " + mEntry);
            String oldPermissions = mEntry.get("unix.mode");
            System.out.println("old permissions: " + oldPermissions);
            String newPermissions = "0775";
            if (newPermissions.equals(oldPermissions)) {
                newPermissions = "0777";
            }
            System.out.println("setting new permissions: " + newPermissions);
            this.conn.setPermissions(newPermissions);
            mEntry = this.conn.doMlst();
            System.out.println("new MlsxEntry: " + mEntry);
            assertTrue(mEntry.get("unix.mode").equals(newPermissions));
            
            this.conn.deleteDirectory();
            
            try {
                this.conn.setPermissions(newPermissions);
                fail("changing permissions on non-existing file should fail");
            } catch(RftTransientException e) {
            } catch(Exception e) {
                fail("expecting RftTransientException and not "
                    + e.getClass().getName());
            }
            
            TestUtil.removeDirIfExists(params1.getSourceHost(),
                params1.getSourcePort(), TestUtil.REMOTE_TEST_DIR,
                this.sourceSubject, credential);
        } catch(Exception e) {
            fail("testPermissions() failed: " + e);
        }
    }

    public void testChangeDirectory() {
        
        try {
            // create a test directory
            TestUtil.createTestDir(params1.getSourceHost(),
                params1.getSourcePort(), this.sourceSubject, credential);
            String dirName = TestUtil.REMOTE_TEST_DIR;
            
            this.conn.changeDirectory(dirName, true);
            this.conn.setPath(TestUtil.REMOTE_TEST_DIR);
            this.conn.deleteDirectory();
            assertFalse(this.conn.exists(dirName));
            
            // change to directory that does not exist.
            // let connection create it.
            this.conn.changeDirectory(dirName, true);
            assertTrue(this.conn.exists(dirName));
            
            // change to directory: no creation. this should cause a
            // transient exception because the directory does not exist
            // anymore
            this.conn.deleteDirectory();
            try {
                this.conn.changeDirectory(dirName, false);
            } catch(RftTransientException e) {
            } catch(Exception e) {
                fail("expecting RftTransientException and not "
                    + e.getClass().getName());
            }
            
            TestUtil.removeDirIfExists(params1.getSourceHost(),
                params1.getSourcePort(), TestUtil.REMOTE_TEST_DIR,
                this.sourceSubject, credential);
        } catch(Exception e) {
            fail("testChangeDirectory() failed: " + e);
        }
    }

    public void testDeleteDirectory() {
        
        try {
            // create a test directory
            TestUtil.createTestDir(params1.getSourceHost(),
                params1.getSourcePort(), this.sourceSubject, credential);
            String dirName = TestUtil.REMOTE_TEST_DIR;
            String fileName = TestUtil.REMOTE_TEST_FILE;
            
            // delete the created directory and verify that the directory
            // and the file do not exist anymore
            try {
                this.conn.setPath(dirName);
                this.conn.deleteDirectory();            
            } catch (Exception e) {
                fail("unable to delete test directory");
            }
            assertFalse(this.conn.exists(dirName) && this.conn.exists(fileName));
            
            try {
                // try to delete it again. this should be ignored
                this.conn.deleteDirectory();
            } catch(Exception e) {
                fail("deletion of a non-existing directory " +
                    dirName + " should not fail");
            }

            TestUtil.removeDirIfExists(params1.getSourceHost(),
                params1.getSourcePort(), TestUtil.REMOTE_TEST_DIR,
                this.sourceSubject, credential);
        } catch(Exception e) {
            fail("testDeleteDirectory() failed: " + e);
        }
    }
    
    public void testDelete()
    {
        try {
            // create a test directory and a testfile
            TestUtil.createTestDir(params1.getSourceHost(),
                params1.getSourcePort(), this.sourceSubject, credential);
            String fileName = TestUtil.REMOTE_TEST_FILE;
                        
            // delete the created file and verify that the file
            // does not exist anymore
            try {
                this.conn.setPath(fileName);
                this.conn.delete();            
            } catch (Exception e) {
                fail("unable to delete test file");
            }
            assertFalse(this.conn.exists(fileName));
            
            try {
                // try to delete the file again. this should be ignored
                this.conn.delete();
            } catch(Exception e) {
                fail("deletion of a non-existing directory " +
                    fileName + " should not fail");
            }
            
            TestUtil.removeDirIfExists(params1.getSourceHost(),
                params1.getSourcePort(), TestUtil.REMOTE_TEST_DIR,
                this.sourceSubject, credential);
        } catch(Exception e) {
            fail("testDeleteDirectory() failed: " + e);
        }
    }

    public void testSetRestartMarkerString() {
        
        // checking invalid restart marker string
        try {
            this.conn.setRestartMarker("1");
            fail("restart marker string is malformed and should fail");
        } catch(RftTransientException e) {
        } catch(Exception e) {
            fail("expecting RftTransientException and not "
                + e.getClass().getName());
        }
        
        // checking valid restart marker string
        try {
            this.conn.setRestartMarker("1-42");
            this.conn.setRestartMarker("1-42,1-345");
        } catch(Exception e) {
            fail("no exception expected for valid restart marker string");
        }
    }

    public void testGetLastUsed() {
        
        // not much to test here. just check that the initial value is 0
        assertTrue(this.conn.getLastUsed() == 0);
    }

    public void testGetPath() {
        
        // not much to test here. just check the initial value
        assertTrue(this.conn.getPath().equals(params1.getSourcePath()));
    }

    public void testSetPath() {
        
        String testPath = "/my/test/Path";
        this.conn.setPath(testPath);
        assertTrue(this.conn.getPath().equals(testPath));
    }

    public void testUpdateLastUsed()
    {
        assertTrue(this.conn.getLastUsed() == 0);
        long tStamp1 = new Date().getTime();
        this.conn.updateLastUsed();
        long lastUsed = this.conn.getLastUsed();
        long tStamp2 = new Date().getTime();
        assertTrue((tStamp1 <= lastUsed) && (lastUsed <= tStamp2));
    }

    public void testUpdateForReuse()
    {
        String currentPath = this.conn.getPath();
        assertTrue(currentPath.equals(params1.getSourcePath()));
        assertFalse(currentPath.equals(params2.getSourcePath()));
        this.conn.updateForReuse(this.params2);
        assertTrue(this.conn.getPath().equals(params2.getSourcePath()));
    }
}
